!!ARBfp1.0
OPTION ARB_precision_hint_fastest;
#OPTION ARB_precision_hint_nicest;

################################################################
# Modified by CnlPepper to implement an advanced planet shader #
################################################################

### IF YOU ARE SEEING THIS TEXT IN THE HW2.LOG FILE THEN THE MEGALITH.FP SHADER HAS FAILED TO LOAD
### THESE SHADERS REQUIRE A GRAPHICS CARD CAPABLE OF AT LEAST SHADER MODEL 2.0B

### NOTE: All shader operations calculated in both TANGENT SPACE and EYE SPACE.

ATTRIB texPos		= fragment.texcoord[0];		# texture coordinates
ATTRIB tem0		= fragment.texcoord[1];		# tangent to eye space transformation matrix row 1
ATTRIB tem1		= fragment.texcoord[2];		# tangent to eye space transformation matrix row 2
ATTRIB tem2		= fragment.texcoord[3];		# tangent to eye space transformation matrix row 3
ATTRIB rawEye		= fragment.texcoord[4];		# interpolated tangent space eye vector
ATTRIB rawVertex	= fragment.texcoord[5];		# interpolated eye space vertex vector

OUTPUT outColour	= result.color;

TEMP tmp1, tmp2, tmp3;
TEMP neg;
TEMP vertex, tsNormal, esNormal, sfLight, sfVertex;
TEMP newPos, thetaPos, phiPos, texDiff, texGlow;
TEMP dotVN, dotLN, thetaR, dotLNn;
TEMP cA, cB, cS, cTheta, cPhi, cSpec;
TEMP gloss, light, scale, diffuse, specular, alpha;

########### TANGENT SPACE ##########

### NORMALISE EYE VECTOR

DP3 vertex, rawEye, rawEye; 
RSQ vertex, vertex.x;
MUL vertex, rawEye, vertex;

### GET NORMAL AND HEIGHT FROM TEXTURE

TEX tmp2, texPos, texture[2], 2D;

### UNPACK AND RE-NORMALISE NORMAL

MAD tmp1, tmp2, {2,2,2,1}, {-1,-1,-1,0};
DP3 tsNormal, tmp1, tmp1; 
RSQ tsNormal, tsNormal.x;
MUL tsNormal, tmp1, tsNormal;

### CALCULATE NEW TEXTURE COORDINATE AND GET SHIFTED TEXTURES (PARALLAX MAPPING)

# shift texture coordinate (offset limited parallax mapping with slope information)
MAD tmp3, tmp2.a, program.local[3], -program.local[4];	# scale and bias defined in shader pass
MUL tmp1, tmp3, -tsNormal.z;
MAD newPos, tmp1, vertex, texPos;

# sample textures
TEX texDiff, newPos, texture[0], 2D;
TEX texGlow, newPos, texture[1], 2D;
TEX tmp2, newPos, texture[2], 2D;

### UNPACK AND RE-NORMALISE NEW NORMAL

MAD tmp1, tmp2, 2, -1;
DP3 tsNormal, tmp1, tmp1; 
RSQ tsNormal, tsNormal.x;
MUL tsNormal, tmp1, tsNormal;

### CONVERT NORMAL FROM TANGENT SPACE TO EYE SPACE

DP3 esNormal.x, tem0, tsNormal; 
DP3 esNormal.y, tem1, tsNormal; 
DP3 esNormal.z, tem2, tsNormal; 

########### EYE SPACE ##########

### RE-NORMALISE VERTEX VECTOR

DP3 vertex, rawVertex, rawVertex; 
RSQ vertex, vertex.x;
MUL vertex, rawVertex, vertex;

### SAMPLE COMMON CONSTANTS/FLAGS

# get A and B constants
TEX cA, {0.00586, 0.01172, 0, 0}, texture[3], 2D;	# pixel (1,1)
TEX cB, {0.01764, 0.01172, 0, 0}, texture[3], 2D;	# pixel (4,1)

# calculate thetaR texture coordinate (unoptimised)
#DP3 dotVN, -vertex, esNormal;
#MAX tmp1, dotVN, 0;
#MAD tmp2, -0.0187293, tmp1, 0.0742610;
#MAD tmp2, tmp2, tmp1, -0.2121144;
#MAD tmp2, tmp2, tmp1, 1.5707288;
#SUB tmp3, 1, tmp1;
#RSQ tmp3.x, tmp3.x;
#RCP tmp3.x, tmp3.x;
#MUL thetaR, tmp2, tmp3.x;
#MUL thetaR, thetaR, 0.63661977; 			# 2/pi
#MAD thetaPos.y, thetaR, 0.953125, 0.035156;		# pixels 5 to 126

# get specular modifier (unoptimised)
#MAD tmp1, thetaR, {0, 0.953125, 0, 0}, {0.00586, 0.035156, 0 ,0};	# pixel (1,5 to 126)
#TEX cSpec, tmp1, texture[3], 2D;

# calculate thetaR texture coordinate and get specular modifier (optimised)
DP3 dotVN, -vertex, esNormal;
MAX tmp1, dotVN, 0;
MAD tmp2, -0.0113645, tmp1, 0.0450600;
MAD tmp2, tmp2, tmp1, -0.1287064;
MAD tmp2, tmp2, tmp1, 0.9530840;
SUB tmp3, 1, tmp1;
RSQ tmp3.x, tmp3.x;
RCP tmp3.x, tmp3.x;
MUL tmp1, tmp2, tmp3.x;
MAD thetaPos.xy, tmp1, {0,1,0,0}, {0.00586, 0.035156, 0 ,0};	# pixel (1,5 to 126)
TEX cSpec, thetaPos, texture[3], 2D;

# calculate projection of view vector on surface
MUL tmp1, esNormal, dotVN;
SUB tmp1, -vertex, tmp1;
DP3 sfVertex, tmp1, tmp1;
ADD sfVertex, sfVertex, 0.001;				# avoid singularity
RSQ sfVertex, sfVertex.x;
MUL sfVertex, tmp1, sfVertex;

### CALCULATE GLOSSINESS FOR SPECULAR

POW tmp1, texGlow.r, {3}.x;
MAD gloss, tmp1, program.local[5], program.local[6];

### CALCULATE LIGHTING FOR LIGHT 0

# get light intensity multiplier
TEX scale, {0.02941, 0.01172, 0, 0}, texture[3], 2D;	# pixel (7,1)

# normalise light vector
DP3 light, state.light[0].position, state.light[0].position; 
RSQ light, light.x;
MUL light, state.light[0].position, light;

# calculate thetaI texture coordinate (unoptimised)
#DP3 dotLN, light, esNormal;
#SLT neg, dotLN, 0;
#MAD neg, neg, -2, 1;
#ABS tmp1, dotLN;
#MAD tmp2, -0.0187293, tmp1,  0.0742610;
#MAD tmp2, tmp2, tmp1, -0.2121144;
#MAD tmp2, tmp2, tmp1, 1.5707288;
#SUB tmp3, 1, tmp1;
#RSQ tmp3.x, tmp3.x;
#RCP tmp3.x, tmp3.x;
#MAD thetaPos.x, tmp2, -tmp3.x, 1.570796;
#MAD thetaPos.x, -thetaPos.x, neg, 1.570796;
#MUL thetaPos.x, thetaPos.x, 0.318309;			# 1/pi
#MAD thetaPos.x, thetaPos.x, 0.976563, 0.017578;	# pixels 5 to 254

# calculate thetaI texture coordinate (optimised)
DP3 dotLN, light, esNormal;
SLT neg, dotLN, 0;
MAD neg, neg, -2, 1;
ABS tmp1, dotLN;
MAD tmp2, -0.0058220, tmp1,  0.0230839;
MAD tmp2, tmp2, tmp1, -0.0659355;
MAD tmp2, tmp2, tmp1, 0.4882591;
SUB tmp3, 1, tmp1;
RSQ tmp3.x, tmp3.x;
RCP tmp3.x, tmp3.x;
MAD thetaPos.x, tmp2, -tmp3.x, 0.4882800;
MAD thetaPos.x, -thetaPos.x, neg, 0.4882800;
ADD thetaPos.x, thetaPos.x, 0.017578;			# pixels 5 to 254

# look-up theta
# CONSIDER ADDING LINEAR INTERPOLATION OF TABLE VALUES
TEX cTheta, thetaPos, texture[3], 2D;		

# calculate projection of light vector on surface
MUL dotLNn, esNormal, dotLN;
SUB tmp1, light, tmp1;
DP3 sfLight, tmp1, tmp1;
ADD sfLight, sfLight, 0.001;				# avoid singularity
RSQ sfLight, sfLight.x;
MUL sfLight, tmp1, sfLight;

# calculate phi texture coordinate (unoptimised)
#DP3 tmp1, sfLight, sfVertex;
#SLT neg, tmp1, 0;
#MAD neg, neg, -2, 1;
#ABS tmp1, tmp1;
#MAD tmp2, -0.0187293, tmp1,  0.0742610;
#MAD tmp2, tmp2, tmp1, -0.2121144;
#MAD tmp2, tmp2, tmp1, 1.5707288;
#SUB tmp3, 1, tmp1;
#RSQ tmp3.x, tmp3.x;
#RCP tmp3.x, tmp3.x;
#MAD phiPos.x, tmp2, -tmp3.x, 1.570796;
#MAD phiPos.x, -phiPos.x, neg, 1.570796;
#MUL phiPos.x, phiPos.x, 0.318309;			# 1/pi
#MAD phiPos.x, phiPos.x, 0.976563, 0.017578;		# pixels 5 to 254
#MOV phiPos.y, 0.01172;					# pixel 1

# calculate phi texture coordinate (optimised)
DP3 tmp1, sfLight, sfVertex;
SLT neg, tmp1, 0;
MAD neg, neg, -2, 1;
ABS tmp1, tmp1;
MAD tmp2, -0.0058220, tmp1,  0.0230839;
MAD tmp2, tmp2, tmp1, -0.0659355;
MAD tmp2, tmp2, tmp1, 0.4882591;
SUB tmp3, 1, tmp1;
RSQ tmp3.x, tmp3.x;
RCP tmp3.x, tmp3.x;
MAD phiPos.x, tmp2, -tmp3.x, 0.4882800;
MAD phiPos.x, -phiPos.x, neg, 0.4882800;
ADD phiPos.x, phiPos.x, 0.017578;			# pixels 5 to 254
MOV phiPos.y, 0.01172;					# pixel 1

# look-up phi
TEX cPhi, phiPos, texture[3], 2D;

# calculate diffuse shading
MUL_SAT tmp1, dotLN, cA.a;
MUL diffuse, cPhi, cB.a;
MAD diffuse, diffuse, cTheta, tmp1;
MUL diffuse, diffuse, state.light[0].diffuse;
MUL diffuse, diffuse, scale.r;

# calculate specular
MAD tmp2, dotLNn, -2, light;
DP3_SAT tmp1, tmp2, vertex;				# prevent negative values
POW tmp1, tmp1.x, gloss.x;
MUL specular, tmp1, texGlow.b;
MUL specular, specular, state.light[0].diffuse;
MUL specular, specular, cSpec;
MUL specular, specular, scale.r;

# calculate alpha
MIN alpha, cTheta.a, cPhi.a;

### CALCULATE LIGHTING FOR LIGHT 1

# get light intensity multiplier
TEX scale, {0.04117, 0.01172, 0, 0}, texture[3], 2D;	# pixel (10,1)

# normalise light vector
DP3 light, state.light[1].position, state.light[1].position; 
RSQ light, light.x;
MUL light, state.light[1].position, light;

# calculate thetaI texture coordinate (unoptimised)
#DP3 dotLN, light, esNormal;
#SLT neg, dotLN, 0;
#MAD neg, neg, -2, 1;
#ABS tmp1, dotLN;
#MAD tmp2, -0.0187293, tmp1,  0.0742610;
#MAD tmp2, tmp2, tmp1, -0.2121144;
#MAD tmp2, tmp2, tmp1, 1.5707288;
#SUB tmp3, 1, tmp1;
#RSQ tmp3.x, tmp3.x;
#RCP tmp3.x, tmp3.x;
#MAD thetaPos.x, tmp2, -tmp3.x, 1.570796;
#MAD thetaPos.x, -thetaPos.x, neg, 1.570796;
#MUL thetaPos.x, thetaPos.x, 0.318309;			# 1/pi
#MAD thetaPos.x, thetaPos.x, 0.976563, 0.017578;	# pixels 5 to 254

# calculate thetaI texture coordinate (optimised)
DP3 dotLN, light, esNormal;
SLT neg, dotLN, 0;
MAD neg, neg, -2, 1;
ABS tmp1, dotLN;
MAD tmp2, -0.0058220, tmp1,  0.0230839;
MAD tmp2, tmp2, tmp1, -0.0659355;
MAD tmp2, tmp2, tmp1, 0.4882591;
SUB tmp3, 1, tmp1;
RSQ tmp3.x, tmp3.x;
RCP tmp3.x, tmp3.x;
MAD thetaPos.x, tmp2, -tmp3.x, 0.4882800;
MAD thetaPos.x, -thetaPos.x, neg, 0.4882800;
ADD thetaPos.x, thetaPos.x, 0.017578;			# pixels 5 to 254

# look-up theta
# CONSIDER ADDING LINEAR INTERPOLATION OF TABLE VALUES
TEX cTheta, thetaPos, texture[3], 2D;		

# calculate projection of light vector on surface
MUL dotLNn, esNormal, dotLN;
SUB tmp1, light, tmp1;
DP3 sfLight, tmp1, tmp1;
ADD sfLight, sfLight, 0.001;				# avoid singularity
RSQ sfLight, sfLight.x;
MUL sfLight, tmp1, sfLight;

# calculate phi texture coordinate (unoptimised)
#DP3 tmp1, sfLight, sfVertex;
#SLT neg, tmp1, 0;
#MAD neg, neg, -2, 1;
#ABS tmp1, tmp1;
#MAD tmp2, -0.0187293, tmp1,  0.0742610;
#MAD tmp2, tmp2, tmp1, -0.2121144;
#MAD tmp2, tmp2, tmp1, 1.5707288;
#SUB tmp3, 1, tmp1;
#RSQ tmp3.x, tmp3.x;
#RCP tmp3.x, tmp3.x;
#MAD phiPos.x, tmp2, -tmp3.x, 1.570796;
#MAD phiPos.x, -phiPos.x, neg, 1.570796;
#MUL phiPos.x, phiPos.x, 0.318309;			# 1/pi
#MAD phiPos.x, phiPos.x, 0.976563, 0.017578;		# pixels 5 to 254
#MOV phiPos.y, 0.01172;					# pixel 1

# calculate phi texture coordinate (optimised)
DP3 tmp1, sfLight, sfVertex;
SLT neg, tmp1, 0;
MAD neg, neg, -2, 1;
ABS tmp1, tmp1;
MAD tmp2, -0.0058220, tmp1,  0.0230839;
MAD tmp2, tmp2, tmp1, -0.0659355;
MAD tmp2, tmp2, tmp1, 0.4882591;
SUB tmp3, 1, tmp1;
RSQ tmp3.x, tmp3.x;
RCP tmp3.x, tmp3.x;
MAD phiPos.x, tmp2, -tmp3.x, 0.4882800;
MAD phiPos.x, -phiPos.x, neg, 0.4882800;
ADD phiPos.x, phiPos.x, 0.017578;			# pixels 5 to 254
MOV phiPos.y, 0.01172;					# pixel 1

# look-up phi
TEX cPhi, phiPos, texture[3], 2D;

# calculate diffuse shading
MUL_SAT tmp1, dotLN, cA.r;
MUL tmp2, cPhi.b, cB.r;
MAD tmp2, tmp2, cTheta, tmp1;
MUL tmp2, tmp2, state.light[1].diffuse;
MAD diffuse, tmp2, scale.r, diffuse;

# calculate specular
MAD tmp2, dotLNn, -2, light;
DP3_SAT tmp1, tmp2, vertex;				# prevent negative values
POW tmp1, tmp1.x, gloss.x;
MUL tmp1, tmp1, texGlow.b;
MUL tmp1, tmp1, state.light[1].diffuse;
MUL tmp1, tmp1, cSpec;
MAD specular, tmp1, scale.r, specular;

# calculate alpha
MIN alpha, alpha, cTheta.a;
MIN alpha, alpha, cPhi.a;

### ADD GLOW AND AMBIENT

ADD diffuse, diffuse, texGlow.g;
ADD diffuse, diffuse, state.lightmodel.ambient;

### COMBINE LIGHTING AND SHADING

MUL tmp1, specular, texDiff;
LRP specular, texGlow.a, specular, tmp1;
MAD outColour, texDiff, diffuse, specular;

### COMBINE ALPHA
MIN outColour.a, alpha, texDiff.a;

END
