!!ARBfp1.0
OPTION ARB_precision_hint_fastest;
#OPTION ARB_precision_hint_nicest;

#########################################################
# Modified by CnlPepper to implement per-pixel lighting #
#########################################################

### IF YOU ARE SEEING THIS TEXT IN THE HW2.LOG FILE THEN THE SHIP.FP SHADER HAS FAILED TO LOAD
### THESE SHADERS REQUIRE A GRAPHICS CARD CAPABLE OF AT LEAST SHADER MODEL 2.0B

### NOTE: All shader operations calculated in EYE SPACE.

ATTRIB texPos		= fragment.texcoord[0];		# texture coordinates
ATTRIB rawNormal	= fragment.texcoord[4];		# interpolated eye space normal
ATTRIB rawVertex	= fragment.texcoord[5];		# interpolated eye space vertex position
ATTRIB lightTypeA	= fragment.texcoord[6];		# flags the type of light for lights 0-3 (0=directional, 1=point)
ATTRIB lightTypeB	= fragment.texcoord[7];		# flags the type of light for lights 4-7 (0=directional, 1=point)

OUTPUT outColour	= result.color;

TEMP temp;
TEMP normal, vertex;
TEMP texDiff, texGlow;
TEMP distA, distB;
TEMP attncol;
TEMP light, dotLN, reflection;
TEMP gloss, diffuse, specular, final, fogColour;
TEMP colBase, colStripe, wBase, wStripe, colour;

### RE-NORMALISE SURFACE NORMAL

DP3 normal, rawNormal, rawNormal; 
RSQ normal, normal.x;
MUL normal, rawNormal, normal;

### RE-NORMALISE EYE VECTOR

DP3 vertex, rawVertex, rawVertex; 
RSQ vertex, vertex.x;
MUL vertex, rawVertex, vertex;

### SAMPLE TEXTURES

TEX texDiff, texPos, texture[0], 2D;
TEX texGlow, texPos, texture[1], 2D;

### CALCULATE GLOSSINESS FROM TEXTURE

POW temp, texGlow.r, {3}.x;
MAD gloss, temp, program.local[5], program.local[6];

### CALCULATE LIGHTING FOR LIGHT 0 (DIFFUSE OR POINT)

# attenuation
SUB temp, rawVertex, state.light[0].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[0].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[0].diffuse, temp;

# re-normalise light vector
MUL temp, rawVertex, lightTypeA.x;			# set according to type of light
SUB temp, state.light[0].position, temp;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MUL specular, temp, attncol;

### CALCULATE LIGHTING FOR LIGHT 1 (DIFFUSE OR POINT, SELECTABLE EXTERNALLY)

# attenuation
SUB temp, rawVertex, state.light[1].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[1].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[1].diffuse, temp;
MAX temp, lightTypeA.y, program.local[4].w; 
MUL attncol, attncol, temp; 

# re-normalise light vector
MUL temp, rawVertex, lightTypeA.y;			# set according to type of light
SUB temp, state.light[1].position, temp;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 2 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[2].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[2].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[2].diffuse, temp;
MUL attncol, attncol, lightTypeA.z;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[2].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 3 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[3].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[3].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[3].diffuse, temp;
MUL attncol, attncol, lightTypeA.w;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[3].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 4 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[4].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[4].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[4].diffuse, temp;
MUL attncol, attncol, lightTypeB.x;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[4].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 5 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[5].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[5].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[5].diffuse, temp;
MUL attncol, attncol, lightTypeB.y;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[5].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 6 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[6].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[6].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[6].diffuse, temp;
MUL attncol, attncol, lightTypeB.z;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[6].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 7 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[7].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[7].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[7].diffuse, temp;
MUL attncol, attncol, lightTypeB.w;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[7].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, normal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, vertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### ADD GLOW

MAD diffuse, texGlow.g, program.local[0], diffuse;	# scale intensity by external glow colour multiplier

### ADD AMBIENT

ADD diffuse, diffuse, state.lightmodel.ambient;

### GENERATE PRE-SHADING SURFACE COLOUR FROM BASE AND TEAM STRIPE COLOURS

# make darker
ADD temp, texDiff, 0.5;
MUL colBase, temp, program.local[1];
MUL colStripe, temp, program.local[2];

# make lighter
SUB temp, texDiff, 0.5;
ADD colBase, temp, colBase;
ADD colStripe, temp, colStripe;

# compute amount of team colour needed
SUB wBase, 1, texDiff.a;
SUB wStripe, 1, texGlow.a;

# average the team colour and base texture
LRP temp.rgb, wBase, colBase, texDiff;
LRP colour, wStripe, colStripe, temp;

# switch colour plain texture if this is the alpha shader pass
LRP colour, program.local[7].w, texDiff, colour;

### COMBINE LIGHTING AND SHADING

MAD final, colour, diffuse, specular;

### APPLY FOG

LRP outColour, program.local[3].a, program.local[3], final;

### EXPORT ALPHA AS NECESSARY

LRP outColour.a, program.local[7].w, texDiff.a, 1;

END

