!!ARBfp1.0
OPTION ARB_precision_hint_fastest;

###############################################################################
# Modified by CnlPepper to implement per-pixel lighting with parallax mapping #
###############################################################################

### NOTE: All shader operations calculated in TANGENT SPACE.

ATTRIB texPos		= fragment.texcoord[0];			# texture coordinates
ATTRIB lightDir		= fragment.texcoord[2];			# texture coordinates
ATTRIB eyeVec		= fragment.texcoord[3];			# texture coordinates

PARAM lightDiff		= state.lightprod[0].diffuse;		# main light diffuse colour product (multiplied by diffuse material property in .st file)
PARAM lightSpec		= state.lightprod[0].specular;		# main light specular colour product (multiplied by specular material property in .st file)
PARAM materialSpecExp	= state.material.shininess;		# material shininess defined in .st file
PARAM misc		= { 0, 0.5, 1.0, 2.0 };

OUTPUT outColour	= result.color;

TEMP temp;
TEMP newPos;
TEMP texDiff, texGlow, texNorm;
TEMP eye, reflection, normal, height, light, gloss;
TEMP dotLN;
TEMP diffuse, specular;
TEMP colBase, colStripe, wBase, wStripe, colour;

### RE-NORMALISE EYE VECTOR

DP3 eye, eyeVec, eyeVec; 
RSQ eye, eye.x;
MUL eye, eyeVec, eye;

### RE-NORMALISE LIGHT VECTOR

DP3 light, lightDir, lightDir; 
RSQ light, light.x;
MUL light, lightDir, light;

### GET NORMAL AND HEIGHT FROM TEXTURE AND UNPACK NORMAL

TEX texNorm, texPos, texture[3], 2D;
MOV height, texNorm.a;
MAD temp, texNorm, 2, -1;

### RE-NORMALISE NORMAL

DP3 normal, temp, temp; 
RSQ normal, normal.x;
MUL normal, temp, normal;

### CALCULATE NEW TEXTURE COORDINATE AND GET SHIFTED TEXTURES (PARALLAX MAPPING)

# shift texture coordinate (offset limited parallax mapping with slope information)
MAD height, height, program.local[2], -program.local[3];	# scale and bias defined in shader pass
MUL temp, height, -normal.z;
MAD newPos, temp, eye, texPos;

# sample textures
TEX texGlow, newPos, texture[1], 2D;
TEX texDiff, newPos, texture[2], 2D;
TEX texNorm, newPos, texture[3], 2D;

### UNPACK AND RE-NORMALISE NEW NORMAL

MAD temp, texNorm, 2, -1;
DP3 normal, temp, temp; 
RSQ normal, normal.x;
MUL normal, temp, normal;

### CALCULATE DIFFUSE LIGHTING

DP3 dotLN, light, normal;
MUL_SAT diffuse, dotLN, lightDiff;		# prevent negative values

### CALCULATE SURFACE SPECULAR LIGHTING CONTRIBUTION (FULL PHONG MODEL)

# calculate reflection vector
MUL temp, normal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# calculate glossiness from texture value
POW temp, texGlow.r, {3}.x;
MAD gloss, temp, program.local[4], program.local[5];

# calculate specular
DP3_SAT temp, reflection, eye;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MUL specular, temp, lightSpec;

### GENERATE PRE-SHADING SURFACE COLOUR FROM BASE AND TEAM STRIPE COLOURS

# make darker
ADD temp, texDiff, misc.y;
MUL colBase, temp, program.local[0];
MUL colStripe, temp, program.local[1];

# make lighter
SUB temp, texDiff, misc.y;
ADD colBase, temp, colBase;
ADD colStripe, temp, colStripe;

# compute amount of team colour needed
SUB wBase, misc.z, texDiff.a;
SUB wStripe, misc.z, texGlow.a;

# average the team colour and base texture
LRP temp.rgb, wBase, colBase, texDiff;
LRP colour, wStripe, colStripe, temp;

# switch colour plain texture if this is the alpha shader pass
LRP colour, program.local[6].w, texDiff, colour;

### COMBINE LIGHTING AND SHADING

MAD outColour, colour, diffuse, specular;

### EXPORT ALPHA AS NECESSARY

TEX temp, texPos, texture[2], 2D;	# unparallaxed alpha
MUL outColour.a, temp.a, program.local[6].w;

END
