!!ARBfp1.0
OPTION ARB_precision_hint_fastest;
#OPTION ARB_precision_hint_nicest;

###############################################################################
# Modified by CnlPepper to implement per-pixel lighting with parallax mapping #
###############################################################################

### IF YOU ARE SEEING THIS TEXT IN THE HW2.LOG FILE THEN THE THRUSTERGLOWPASS.FP SHADER HAS FAILED TO LOAD
### THESE SHADERS REQUIRE A GRAPHICS CARD CAPABLE OF AT LEAST SHADER MODEL 2.0B

### NOTE: All shader operations calculated in both TANGENT SPACE and EYE SPACE.

ATTRIB texPos		= fragment.texcoord[0];		# texture coordinates
ATTRIB tem0		= fragment.texcoord[1];		# tangent to eye space transformation matrix row 1
ATTRIB tem1		= fragment.texcoord[2];		# tangent to eye space transformation matrix row 2
ATTRIB tem2		= fragment.texcoord[3];		# tangent to eye space transformation matrix row 3
ATTRIB rawEye		= fragment.texcoord[4];		# interpolated tangent space eye vector
ATTRIB rawVertex	= fragment.texcoord[5];		# interpolated eye space vertex vector
ATTRIB lightTypeA	= fragment.texcoord[6];		# flags the type of light for lights 0-3 (0=directional, 1=point)
ATTRIB lightTypeB	= fragment.texcoord[7];		# flags the type of light for lights 4-7 (0=directional, 1=point)

OUTPUT outColour	= result.color;

TEMP temp;
TEMP newPos;
TEMP texDiff, texGlow, texNorm;
TEMP distA, distB;
TEMP tsEye, reflection, tsNormal, esNormal, height, gloss, light, attncol, esVertex;
TEMP dotLN;
TEMP diffuse, specular;
TEMP colBase, colStripe, wBase, wStripe, colour;

########### TANGENT SPACE ##########

### NORMALISE EYE VECTOR

DP3 tsEye, rawEye, rawEye; 
RSQ tsEye, tsEye.x;
MUL tsEye, rawEye, tsEye;

### GET NORMAL AND HEIGHT FROM TEXTURE

TEX texNorm, texPos, texture[2], 2D;
MOV height, texNorm.a;

### UNPACK AND RE-NORMALISE NORMAL

MAD temp, texNorm, 2, -1;
DP3 tsNormal, temp, temp; 
RSQ tsNormal, tsNormal.x;
MUL tsNormal, temp, tsNormal;

### CALCULATE NEW TEXTURE COORDINATE AND GET SHIFTED TEXTURES (PARALLAX MAPPING)

# shift texture coordinate (offset limited parallax mapping with slope information)
MAD height, height, program.local[3], -program.local[4];	# scale and bias defined in shader pass
MUL temp, height, -tsNormal.z;
MAD newPos, temp, tsEye, texPos;

# sample textures
TEX texDiff, newPos, texture[0], 2D;
TEX texGlow, newPos, texture[1], 2D;
TEX texNorm, newPos, texture[2], 2D;

### UNPACK AND RE-NORMALISE NEW NORMAL

MAD temp, texNorm, 2, -1;
DP3 tsNormal, temp, temp; 
RSQ tsNormal, tsNormal.x;
MUL tsNormal, temp, tsNormal;

### CONVERT NORMAL FROM TANGENT SPACE TO EYE SPACE

DP3 esNormal.x, tem0, tsNormal; 
DP3 esNormal.y, tem1, tsNormal; 
DP3 esNormal.z, tem2, tsNormal; 

########### EYE SPACE ##########

### RE-NORMALISE VERTEX VECTOR

DP3 esVertex, rawVertex, rawVertex; 
RSQ esVertex, esVertex.x;
MUL esVertex, rawVertex, esVertex;

### CALCULATE GLOSSINESS FROM TEXTURE

POW temp, texGlow.r, {3}.x;
MAD gloss, temp, program.local[5], program.local[6];

### CALCULATE LIGHTING FOR LIGHT 0 (DIRECTIONAL OR POINT)

# attenuation
SUB temp, rawVertex, state.light[0].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[0].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[0].diffuse, temp;

# re-normalise light vector
MUL temp, rawVertex, lightTypeA.x;			# set according to type of light
SUB temp, state.light[0].position, temp;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MUL specular, temp, attncol;

### CALCULATE LIGHTING FOR LIGHT 1 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[1].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[1].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[1].diffuse, temp;
MUL attncol, attncol, lightTypeA.y;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[1].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 2 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[2].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[2].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[2].diffuse, temp;
MUL attncol, attncol, lightTypeA.z;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[2].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 3 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[3].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[3].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[3].diffuse, temp;
MUL attncol, attncol, lightTypeA.w;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[3].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 4 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[4].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[4].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[4].diffuse, temp;
MUL attncol, attncol, lightTypeB.x;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[4].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 5 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[5].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[5].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[5].diffuse, temp;
MUL attncol, attncol, lightTypeB.y;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[5].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 6 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[6].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[6].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[6].diffuse, temp;
MUL attncol, attncol, lightTypeB.z;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[6].position, rawVertex; #temp;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### CALCULATE LIGHTING FOR LIGHT 7 (POINT ONLY)

# attenuation
SUB temp, rawVertex, state.light[7].position;
DP3 distA, temp, temp; 
RSQ distB, distA.x;
DST temp, distA, distB;
DP3 temp, temp, state.light[7].attenuation;
RCP temp, temp.x;

# attenuate light colour
MUL attncol, state.light[7].diffuse, temp;
MUL attncol, attncol, lightTypeB.w;			# disable output if directional light

# re-normalise light vector
SUB temp, state.light[7].position, rawVertex;
DP3 light, temp, temp; 
RSQ light, light.x;
MUL light, temp, light;

# diffuse with attenuation
DP3 dotLN, light, esNormal;
MUL_SAT temp, dotLN, attncol;				# prevent negative values
ADD diffuse, diffuse, temp;	

# calculate reflection vector
MUL temp, esNormal, dotLN;
MUL temp, temp, 2;
SUB reflection, light, temp;

# specular with attenuation
DP3_SAT temp, reflection, esVertex;			# prevent negative values
POW temp, temp.x, gloss.x;
MUL temp, temp, texGlow.b;
MAD specular, temp, attncol, specular;

### ADD GLOW

MAD diffuse, texGlow.g, program.local[0], diffuse;	# scale intensity by external glow colour multiplier

### ADD AMBIENT

ADD diffuse, diffuse, state.lightmodel.ambient;

### GENERATE PRE-SHADING SURFACE COLOUR FROM BASE AND TEAM STRIPE COLOURS

# make darker
ADD temp, texDiff, 0.5;
MUL colBase, temp, program.local[1];
MUL colStripe, temp, program.local[2];

# make lighter
SUB temp, texDiff, 0.5;
ADD colBase, temp, colBase;
ADD colStripe, temp, colStripe;

# compute amount of team colour needed
SUB wBase, 1, texDiff.a;
SUB wStripe, 1, texGlow.a;

# average the team colour and base texture
LRP temp.rgb, wBase, colBase, texDiff;
LRP colour, wStripe, colStripe, temp;

# switch colour plain texture if this is the alpha shader pass
LRP colour, program.local[7].w, texDiff, colour;

### COMBINE LIGHTING AND SHADING

MAD outColour, colour, diffuse, specular;

### EXPORT ALPHA AS NECESSARY

LRP outColour.a, program.local[7].w, texDiff.a, 1;

END
