-- ============================================================================
-- Homeworld 2: Area Capture Mod
-- -----------------------------
--
-- Author:Alex "CnlPepper" Meakins
-- Version: 1.0
-- Date: 06/04/2008
-- File: AreaCapture.lua
-- 
-- This file defines area capture Homeworld 2 game type 
--
-- ============================================================================


GUID = { 110, 91, 157, 190, 18, 23, 250, 78, 144, 20, 41, 246, 181, 128, 214, 12, }
GameRulesName = "Capture"
Description = "Area capture gametype"
Directories = { Levels = "data:LevelData\\Multiplayer\\AreaCapture\\", }
dofilepath("data:scripts/scar/restrict.lua")
dofilepath("data:leveldata/multiplayer/AreaCapture/GameSetupOptionsArea.lua")
dofilepath("data:scripts/scar/AreaCaptureRules.lua")
--dofilepath("data:scripts/Kamikaze.lua")
dofilepath("data:scripts/NoJumping.lua")

-- Race helper code used in restrict and shipbonuses
dofilepath("data:scripts/race.lua")

dofilepath("data:scripts/scar/scar_util.lua")
dofilepath("data:scripts/hw2bsg_util.lua")
dofilepath("data:scripts/RandomMusicTrack.lua")

function SetStartingFleetSuffixFromOption()
    local startingFleetSuffix = GetGameSettingAsString("startingfleetsuffix") -- doesn't work on OSX
	if (startingFleetSuffix) then                  
        print("StaringFleetSuffix: " .. startingFleetSuffix)
	    SetStartFleetSuffix(startingFleetSuffix)
	end
end

Events = {}
Events.endGame =  { 
	{ 
		{ "wID = Wait_Start(5)", "Wait_End(wID)", }, 
	}, 
}

victOwnership = 0	-- holds the number of control points that need to be captured to trigger a control point ownership victory

function OnInit()

	-- set starting fleet
	local suffix = GetGameSettingAsString("startingfleet")
	 SetStartingFleetSuffixFromOption()

	-- set area capture values from interface settings
	cpCaptureTime = GetGameSettingAsNumber("capturetime") + 20
	cpAwardRUValue = GetGameSettingAsNumber("ruawardvalue") + 10
	cpAwardStationRespawnDelay = GetGameSettingAsNumber("stationrespawn") + 30
	cpAwardBabylon4RespawnDelay = GetGameSettingAsNumber("stationrespawn") + 30
	cpAwardBabylon5RespawnDelay = GetGameSettingAsNumber("stationrespawn") + 30
	cpAwardShipInitialDelay = GetGameSettingAsNumber("shipspawninitial") + 25
	cpAwardShipSpawnPeriod = GetGameSettingAsNumber("shipspawnperiod") + 60

	PickRandomMusicTrackRule() -- PickRandomMusicTrack.lua imported by restirct.lua
	
	Player_SetDefaultShipTactic(0, AggressiveTactics)
	Player_SetDefaultShipTactic(1, AggressiveTactics)
	
	Rule_Add("Rule_Binding")
	Rule_Add("Rule_NoJumping")
	Rule_Add("Rule_Text")

	-- init game rules
	MPRestrict()
	Rule_Add("MainRule")
	AreaCaptureInit()
	-- set victory condition variables
	victOwnership = GetGameSettingAsNumber("vownership") * cpTotal

end

function Rule_Text()
	LocationCard("No Jumping for 5 Minutes", 14)
	Rule_Remove("Rule_Text")
end

function MainRule()

	local playerCount = Universe_PlayerCount() - 1

	-- kill players that loose all build ships and reset their control points
	for playerIndex=0, playerCount do
		if (Player_IsAlive(playerIndex) == 1) then

			-- if all build capable ships are gone, so is the player!
			if (Player_HasShipWithBuildQueue(playerIndex) == 0) then

				killPlayer(playerIndex)

			end 
		end 
	end 

	-- reset player control point totals
	local playerCPCount = {}		
	for playerIndex=0, playerCount do
	
		playerCPCount[playerIndex] = 0

	end
	
	-- accumulate control point ownership totals
	for c=1, cpTotal do
	
		if (cpOwnership[c] >= 0) then
		
			playerCPCount[cpOwnership[c]] = playerCPCount[cpOwnership[c]] + 1

		end
	end
	
	-- check control point victory condition
	for playerIndex=0, playerCount do

		if (Player_IsAlive(playerIndex) == 1) then
		
			-- get alliance totals
			local teamCPCount = playerCPCount[playerIndex]
			for otherPlayerIndex = 0, playerCount do

				if (otherPlayerIndex ~= playerIndex) and (AreAllied(playerIndex, otherPlayerIndex) == 1) then
				
					teamCPCount = teamCPCount + playerCPCount[otherPlayerIndex]

				end
			end		

			-- check if victory conditions achieved
			if (victOwnership ~= 0) and (teamCPCount >= victOwnership) then

				-- this team is victorious, kill the other players
				killAllNonAlliedPlayers(playerIndex)

			end
		end	
	end

	-- check elimination victory condition
	local numAlive = 0
	local numEnemies = 0
	local gameOver = 1
	for playerIndex=0, playerCount do
		
		if (Player_IsAlive(playerIndex) == 1) then
		
			for otherPlayerIndex=0, playerCount do
		
				if (AreAllied(playerIndex, otherPlayerIndex) == 0) then
					
					if (Player_IsAlive(otherPlayerIndex) == 1) then
					
						gameOver = 0
					
					else
					
						numEnemies = numEnemies + 1
					
					end 
				end 
			end 

		numAlive = numAlive + 1

		end 
	end 

	if (numEnemies == 0) and (numAlive > 0) then

		gameOver = 0

	end 

	-- have the victory conditions been met?
	if (gameOver == 1) then

		Rule_Add("waitForEnd")
		Event_Start("endGame")
		Rule_Remove("MainRule")

	end 
end

function killPlayer(player)

	Player_Kill(player)
	AreaCapturePurgePlayer(player)
	
end

function killAllNonAlliedPlayers(player)

	for index=0, (Universe_PlayerCount() - 1) do
	
		if (index ~= player) and (AreAllied(index, player) == 0) then
		
			if (Player_IsAlive(player) == 1) then
			
				killPlayer(index)
				
			end		
		end
	end
end

function waitForEnd()

	if  Event_IsDone("endGame") then

		setGameOver()
		Rule_Remove("waitForEnd")

	end 
end
