-- ============================================================================
-- Homeworld 2: Area Capture Mod
-- -----------------------------
--
-- Author:Alex " CnlPepper" Meakins
-- Version: 1.0
-- Date: 31/07/2008
-- File: AreaCaptureAwardBabylon5.lua
-- 
-- This file defines the Babylon 5 station award. On a control point capture
-- Babylon 5 is spawned. If the station is destroyed a counter is started which
-- counts down the time for the station to respawn. The station will only
-- respawn provided the control point is still owned by the player and that
-- there are no other stations present (ie it won't respawn if the station was 
-- captured, and not destroyed). The time until the station respawns is
-- displayed using the award progess bar in the UI.
--
-- This award must not be used in conjunction with other awards that use the
-- award progress bar.
--
-- ============================================================================

cpAwardBabylon5RespawnDelay = 60	-- WARNING: this value is now overwritten by the gui value setting in AreaCapture.lua OnInit()

-- hold the station to spawn for each race
cpAwardStationList5 =
{
	race1 = "hur_parnassus",
	--race2 = "VGR_SHIPYARD",
	race3 = "cyr_patrolstar",
	--race4 = "EAR_ORIONSTATION",
}


cpAwardBabylon5IsSet = {}		-- cache if the award is set for each control point
cpAwardBabylon5SpawnTime = {}		-- station spawn delay timers, holds the time until the next station spawn for each player
cpAwardBabylon5IsLost = {}		-- holds if the station was detected as lost by previous check, only if initially declared lost and timer has elapsed can the station be respawned
cpAwardStationListString5 = ""		-- ship list string generated from cpAwardStationList

function AreaCaptureAwardBabylon5Init()
	
	-- cache if award is set, reset progress bar, initialise spawn timers and lost state
	for c=1,cpTotal do
	
		for i,award in cpAwards[c] do
				
			-- find first instance of station award
			cpAwardBabylon5IsSet[c] = 0
			if (award == "Babylon5") then

				-- init cache, station lost array and UI
				cpAwardBabylon5IsSet[c] = 1
				cpAwardBabylon5IsLost[c] = 0
				AreaCaptureUIClock(c, 100, "Award")					

				-- set initial spawn time to current time
				cpAwardBabylon5SpawnTime[c] = {}
				for j=0,(Universe_PlayerCount()-1) do
				
					cpAwardBabylon5SpawnTime[c][j] = Universe_GameTime()

				end				

				-- award found, escape
				break
						
			end
		end
	end
-- assemble station list string
	cpAwardStationListString5 = ""
	for i,v in cpAwardStationList5 do
	
		cpAwardStationListString5 = cpAwardStationListString5 .. v .. ", "

	end
end

function AreaCaptureAwardBabylon5Rule()

	-- periodically call award station to check whether the station is missing and needs to be respawned
	for c=1,cpTotal do

		if (cpAwardBabylon5IsSet[c] == 1) then

			if (cpOwnership[c] >= 0) then
				
				if (cpAwardBabylon5IsLost[c] == 0) then
							
					-- check if the station is missing
					if (SobGroup_AreAnyOfTheseTypes("CP"..c, cpAwardStationListString5) == 0) then

						-- set lost status and timer for owning player
						cpAwardBabylon5IsLost[c] = 1
						cpAwardBabylon5SpawnTime[c][cpOwnership[c]] = Universe_GameTime() + cpAwardBabylon5RespawnDelay
					
					end

				else
			
					AreaCaptureAwardBabylon5(cpOwnership[c], c)
			
				end
			end
		
			-- update the progress bar
			AreaCaptureAwardBabylon5UpdateUI(c)

		end
	end
end


function AreaCaptureAwardBabylon5(iPlayer, iControlPoint)

	-- is timer expired
	if ((cpAwardBabylon5SpawnTime[iControlPoint][iPlayer] - Universe_GameTime()) <= 0) then

		-- only spawn if a station is not already present
		if (SobGroup_AreAnyOfTheseTypes("CP"..iControlPoint, cpAwardStationListString5) == 0) then

			-- spawn station
			AreaCaptureAwardBabylon5Spawn(iPlayer, iControlPoint, cpAwardStationList5["race"..Player_GetRace(iPlayer)])	

			-- reset lost status
			cpAwardBabylon5IsLost[iControlPoint] = 0

		end
	end
end


function AreaCaptureAwardBabylon5Spawn(iPlayer, iControlPoint, sShip)


	print("AreaCaptureAwardBabylon5: Spawning EAR_BABYLON5 for player "..iPlayer.." at control point "..iControlPoint)
	SobGroup_SpawnNewShipInSobGroup(iPlayer, sShip, "null", "CP"..iControlPoint, "CP"..iControlPoint.."_Volume")

end

function AreaCaptureAwardBabylon5UpdateUI(iControlPoint)

	if ((cpAwardBabylon5SpawnTime[iControlPoint][FE_GetCurrentPlayerIndex()] - Universe_GameTime()) > 0) then
	
		AreaCaptureUIClock(iControlPoint, (1 - ((cpAwardBabylon5SpawnTime[iControlPoint][FE_GetCurrentPlayerIndex()] - Universe_GameTime()) / cpAwardBabylon5RespawnDelay)) * 100, "Award")

	else
	
		AreaCaptureUIClock(iControlPoint, 100, "Award")
	
	end
end
