-- ============================================================================
-- Homeworld 2: Area Capture Mod
-- -----------------------------
--
-- Author:Alex " CnlPepper" Meakins
-- Version: 1.0
-- Date: 06/04/2008
-- File: AreaCaptureAwardResearch.lua
-- 
-- This file defines the research award. While a control point with this award
-- is owned the player will be granted the ability to build a unique class of
-- ship, provided they already have the required level of technology.
--
-- ============================================================================


cpAwardResearchCount = {}	-- holds the number of research points owned by each player.

function AreaCaptureAwardResearchInit()

	for i=0,(Universe_PlayerCount()-1) do
	
		-- pre-restrict research options
		if (Player_GetRace(i) == 1) then	-- HUR
		
			Player_GrantResearchOption (i, "buildbasestar")
			Player_RestrictBuildOption(i, "hur_basestar_rebel")
		
		elseif (Player_GetRace(i) == 3)	then	-- CYR

			Player_GrantResearchOption (i, "buildkamikaze")
			Player_RestrictBuildOption(i, "cyr_Kamikaze")
		
		end
	
		-- initialise research award ownership counter
		cpAwardResearchCount[i] = 0
	
	end
end


function AreaCaptureAwardResearchEnable(iPlayer)

	-- only give research if not previously given
	if (cpAwardResearchCount[iPlayer] == 0) then

		print("AreaCaptureAwardResearch: Giving research award to player "..iPlayer)
	
		if (Player_GetRace(iPlayer) == 1) then		-- HUR

			Player_UnrestrictBuildOption(iPlayer, "hur_basestar_rebel")

		elseif (Player_GetRace(iPlayer) == 3) then	-- CYR

			Player_UnrestrictBuildOption(iPlayer, "cyr_Kamikaze")

		end
	end

	-- increment award counter
	cpAwardResearchCount[iPlayer] = cpAwardResearchCount[iPlayer] + 1

end



function AreaCaptureAwardResearchDisable(iPlayer)

	-- decrement award counter
	cpAwardResearchCount[iPlayer] = cpAwardResearchCount[iPlayer] - 1
	
	-- remove award if player no longer has any research points
	if (cpAwardResearchCount[iPlayer] == 0) then
	
		print("AreaCaptureAwardResearch: Removing research award from player "..iPlayer)
	
		if (Player_GetRace(iPlayer) == 1) then		-- HUR

			Player_RestrictBuildOption(iPlayer, "hur_basestar_rebel")

		elseif (Player_GetRace(iPlayer) == 3) then	-- CYR

			Player_RestrictBuildOption(iPlayer, "cyr_Kamikaze")

		end
	end
end
